from __future__ import absolute_import, print_function, unicode_literals
import docker
import mock
import pytest

from dockalot.config import Config
from dockalot.docker import parse_args, \
    pull_base_image, make_container, run_command_list, commit_image, \
    tag_image


@pytest.fixture
def mock_docker_client():
    return mock.create_autospec(docker.Client)


@pytest.fixture
def mock_config():
    return Config({'docker': {'base_image': 'junkapotamus:1.0'}})


@pytest.mark.parametrize('args,attrname,expected_value', [
    ([], 'configfile', 'fake_configfile.yml'),
    (['--ask-vault-pass'], 'ansible_args', ['--ask-vault-pass']),
    (['-e', 'v1=a', '-e', 'v2=b'], 'ansible_args',
        ['-e', 'v1=a', '-e', 'v2=b']),
    (['--vault-password-file', 'passwd.txt'], 'ansible_args',
        ['--vault-password-file', 'passwd.txt']),
])
def test_base_parse_args(args, attrname, expected_value):
    """Verify parsing of the base command line args"""
    result = parse_args(args=args + ['fake_configfile.yml'])
    assert getattr(result, attrname) == expected_value


@pytest.mark.parametrize('args,attrname,expected_value', [
    ([], 'pull', False),
    (['--label', 'a=1', '--label', 'b=2'], 'label', ['a=1', 'b=2']),
    (['--pull'], 'pull', True),
    (['-t', 't1', '-t', 't2'], 'tag', ['t1', 't2']),
])
def test_docker_parse_args(args, attrname, expected_value):
    """Verify parsing of the Docker command line args"""
    result = parse_args(args=args + ['fake_configfile.yml'])
    assert getattr(result, attrname) == expected_value


@pytest.mark.parametrize('have_image,always_pull,expect_pull', [
    (False, False, True),
    (True, False, False),
    (True, True, True),
])
def test_pull_base_image(have_image, always_pull, expect_pull,
        mock_config, mock_docker_client):
    mock_config.items['always_pull'] = always_pull
    mock_docker_client.images.return_value = [{'Id': 'abc'}] if have_image \
        else []
    mock_docker_client.pull.return_value = '{"status": "great"}'

    pull_base_image(mock_config, mock_docker_client)

    if expect_pull:
        mock_docker_client.pull.assert_called_with(repository='junkapotamus',
            tag='1.0')
    else:
        mock_docker_client.pull.assert_not_called()


def test_make_container(mock_config, mock_docker_client):
    image_name = 'junkapotamus:1.0'
    container_id = 'abcd'

    mock_docker_client.create_container.return_value = {
        'Id': container_id,
        'Warnings': None,
    }

    assert make_container(mock_config, mock_docker_client) == container_id
    mock_docker_client.create_container.assert_called_with(image_name,
        command='sleep 360000', networking_config=None)
    mock_docker_client.start.assert_called_with(resource_id=container_id)


def test_run_command_list(mock_docker_client):
    fake_container_id = 'abcd'
    fake_exec_id = 'dumbledoor'
    commands = [['echo', 'hello']]

    # Set up the mocks
    mock_docker_client.exec_create.return_value = {'Id': fake_exec_id}
    mock_docker_client.exec_start.return_value = 'hello\n'
    mock_docker_client.exec_inspect.return_value = {'ExitCode': 0}

    # Run and verify the mocks were called properly
    run_command_list(commands, mock_docker_client, fake_container_id)
    mock_docker_client.exec_create.assert_called_with(
        container=fake_container_id, cmd=commands[0])
    mock_docker_client.exec_start.assert_called_with(exec_id=fake_exec_id)
    mock_docker_client.exec_inspect.assert_called_with(exec_id=fake_exec_id)


@pytest.mark.parametrize('docker_config,expected_extra_commands', [
    ({'cmd': ['ls', '-al']}, ['CMD ["ls", "-al"]']),
    ({'entrypoint': ['ls', '-al']}, ['ENTRYPOINT ["ls", "-al"]']),
    ({'expose_ports': [123, 456]}, ['EXPOSE 123', 'EXPOSE 456']),
    ({'volumes': ['/a', '/b']}, ['VOLUME /a', 'VOLUME /b']),
    ({'labels': {'name': 'foo'}}, ['LABEL "name"="foo"']),
    ({'labels': {'desc': 'f\\o"o'}}, ['LABEL "desc"="f\\\\o\\"o"']),
    ({'workdir': '/home'}, ['WORKDIR /home']),
    ({'env': {'PATH': '/bin'}}, ['ENV PATH /bin']),
    ({'env': {'VERSION': '\"foo 1.0\"'}}, ['ENV VERSION \"foo 1.0\"']),
])
def test_commit_image(docker_config, expected_extra_commands,
        mock_config, mock_docker_client):
    container_id = 'fake_container_id'
    image_id = 'fake_image_id'

    mock_config.items['docker'].items.update(docker_config)
    mock_docker_client.commit.return_value = {'Id': image_id}

    assert commit_image(mock_config, mock_docker_client, container_id) == \
        image_id
    mock_docker_client.commit.assert_called_with(container_id,
        changes=expected_extra_commands)


def test_tag_image(mock_docker_client):
    config = Config({'docker': {
        'base_image': 'unused',
        'tags': ['foo:1.0', 'foo:latest']}})
    image_id = 'abcdzyxw'

    tag_image(config, mock_docker_client, image_id)
    mock_docker_client.remove_image.assert_has_calls([
        mock.call(resource_id='foo:1.0'),
        mock.call(resource_id='foo:latest'),
    ])
    mock_docker_client.tag.assert_has_calls([
        mock.call(resource_id=image_id, repository='foo', tag='1.0'),
        mock.call(resource_id=image_id, repository='foo', tag='latest'),
    ])


# vim:set ts=4 sw=4 expandtab:



from __future__ import absolute_import, print_function, unicode_literals
import pytest
import subprocess

from .markers import integration


@pytest.fixture
def test_net(docker_client):
    """Create a user-defined network. Returns the network name."""
    net_name = 'dockalot-inttest'
    docker_client.create_network(net_name, driver='bridge')

    yield net_name

    docker_client.remove_network(net_name)


@integration
def test_build_on_network(basic_config, image_tracker, test_net):
    """Attach the build container to a user-defined network"""
    subprocess.check_call(['dockalot', '--network', test_net, basic_config])
    images = image_tracker.get_image_ids()
    assert len(images) == 1



from __future__ import absolute_import, print_function, unicode_literals
import mock
import pytest

from dockalot.config import ConfigurationError, Config, DockerConfig, \
    string_importer, integer_importer, enum_importer, \
    string_list_importer, integer_list_importer, \
    string_dict_importer


# Class for holding fake command line args parsed from argparse
class FakeArgs(object):
    def __init__(self, args):
        self.ansible_args = args.get('ansible_args', None)
        self.env = args.get('env', None)
        self.label = args.get('label', None)
        self.network = args.get('network', None)
        self.pull = args.get('pull', False)
        self.tag = args.get('tag', None)


@pytest.mark.parametrize('importer,input,expected_value', [
    (string_importer, 'abc', 'abc'),
    (string_importer, 123, '123'),
    (string_importer, True, 'True'),
    (integer_importer, 123, 123),
    (integer_importer, '123', 123),
    (enum_importer(['aaa', 'bbb']), 'aaa', 'aaa'),
    (enum_importer(['aaa', 'bbb']), 'bbb', 'bbb'),
    (enum_importer(['aaa', 'bbb']), 'AAa', 'aaa'),
    (string_list_importer, [], []),
    (string_list_importer, [1, '2', False], ['1', '2', 'False']),
    (integer_list_importer, [], []),
    (integer_list_importer, [1, '2'], [1, 2]),
    (string_dict_importer, {}, {}),
    (string_dict_importer, {'a': 'abcd', 'b': 12}, {'a': 'abcd', 'b': '12'}),
])
def test_importer_valid(importer, input, expected_value):
    """Test low-level importers successfully convert values"""
    assert importer(input, ['test']) == expected_value


@pytest.mark.parametrize('importer,input', [
    (string_importer, ['abcd']),
    (string_importer, {'a': 1}),
    (string_importer, object()),
    (integer_importer, 'abcd'),
    (integer_importer, ['1']),
    (integer_importer, {'a': '1'}),
    (enum_importer(['aaa', 'bbb']), 'xxx'),
    (string_list_importer, 'not_a_list'),
    (string_list_importer, ['a', 'b', []]),
    (integer_list_importer, 'not_a_list'),
    (integer_list_importer, ['abcd']),
    (string_dict_importer, {'a': ['abcd', 'efgh']}),
])
def test_importer_invalid(importer, input):
    """Test low-level importers successfully reject bad values"""
    with pytest.raises(ConfigurationError):
        importer(input, ['test'])


@pytest.mark.parametrize('config_dict,key,expected_value', [
    ({}, 'docker', {}),
    ({'inventory_groups': ['a']}, 'inventory_groups', ['a']),
    ({'preparation_commands': ['xxx']}, 'preparation_commands', ['xxx']),
    ({'cleanup_commands': ['yyy']}, 'cleanup_commands', ['yyy']),
])
@mock.patch('dockalot.config.docker_section_importer', return_value={})
def test_Config_validation(m, config_dict, key, expected_value):
    """Tests successful validation of top-level configuration items"""
    # Add a fake 'docker' section because it is required
    config_dict['docker'] = {}

    cfg = Config(config_dict)
    assert cfg[key] == expected_value


def test_Config_nested_error_message():
    """
    Tests that an error message from a nested config section displays
    like expected.
    """
    config_dict = {
        'docker': {
            'base_image': 'a',
            'workdir': ['not a string'],
        }
    }
    with pytest.raises(ConfigurationError) as e:
        Config(config_dict)
    assert e.value.message.startswith("Configuration value 'docker.workdir' ")


@pytest.mark.parametrize('config_dict,key,expected_value', [
    ({'base_image': 'debian'},
        'base_image', 'debian'),
    ({'cmd': ['bash'], 'base_image': 'debian'},
        'cmd', ['bash']),
    ({'entrypoint': ['/start.sh'], 'base_image': 'debian'},
        'entrypoint', ['/start.sh']),
    ({'env': {'A': 'abc'}, 'base_image': 'debian'},
        'env', {'A': 'abc'}),
    ({'expose_ports': [123, 345], 'base_image': 'debian'},
        'expose_ports', [123, 345]),
    ({'labels': {'a': 'wee', 'b': 2}, 'base_image': 'debian'},
        'labels', {'a': 'wee', 'b': '2'}),
    ({'volumes': ['v1', 'v2'], 'base_image': 'debian'},
        'volumes', ['v1', 'v2']),
    ({'workdir': '/root', 'base_image': 'debian'},
        'workdir', '/root'),
])
def test_DockerConfig_validation(config_dict, key, expected_value):
    """
    Tests successful validation of items in the docker image configuration
    """
    c = DockerConfig(config_dict, prefix=['docker'])
    assert c[key] == expected_value


@pytest.mark.parametrize('config_dict', [
    {},  # Missing 'base_image'
    {'base_image': ['not', 'a', 'string']},
    {'base_image': 'debian', 'cmd': 'not_a_list'},
    {'base_image': 'debian', 'entrypoint': 'not_a_list'},
    {'base_image': 'debian', 'env': 'not_a_dict'},
    {'base_image': 'debian', 'expose_ports': ['nan', 'nan']},
    {'base_image': 'debian', 'labels': 'not_a_dict'},
    {'base_image': 'debian', 'volumes': 'not_a_list'},
])
def test_DockerConfig_validation_failure(config_dict):
    with pytest.raises(ConfigurationError):
        DockerConfig(config_dict, prefix=['docker'])


@pytest.mark.parametrize('config_dict,arg_dict,item_name,expected_value', [
    # Tags specified on the command line replace all tags from the config
    ({'tags': ['a', 'b']}, {}, 'tags', ['a', 'b']),
    ({'tags': ['a', 'b']}, {'tag': ['c']}, 'tags', ['c']),

    # Labels specified on the command line replace all labels in the config
    ({'labels': {'name': 'foo'}}, {}, 'labels', {'name': 'foo'}),
    ({'labels': {'name': 'foo'}}, {'label': ['desc=frob']},
        'labels', {'desc': 'frob'}),
])
def test_DockerConfig_merge_command_line_args(config_dict, arg_dict,
        item_name, expected_value):
    """
    Tests DockerConfig.merge_command_line_args

    With a given configuration in `config_dict` and command line arguments
    in `arg_dict`, after merging in the command line args we'll expect
    the given `item_name` in the configuration to be equal to the
    `expected_value`.
    """
    config_dict['base_image'] = 'debian'  # Required parameter for DockerConfig
    docker_config = DockerConfig(config_dict, prefix=[])
    args = FakeArgs(arg_dict)

    docker_config.merge_command_line_args(args)
    assert docker_config[item_name] == expected_value


@pytest.mark.parametrize('config_dict,arg_dict,item_name,expected_value', [
    ({}, {'network': 'my-net'}, 'build_network', 'my-net'),
    ({}, {'pull': True}, 'always_pull', True),
])
def test_Config_merge_command_line_args(config_dict, arg_dict,
        item_name, expected_value):
    """
    Tests Config.merge_command_line_args
    """
    config_dict['docker'] = {'base_image': 'debian'}
    config = Config(config_dict)
    args = FakeArgs(arg_dict)

    config.merge_command_line_args(args)
    assert config[item_name] == expected_value


# vim:set ts=4 sw=4 expandtab:
